program head;

{  Executable filename: HEAD.EXE                                        }
{                                                                       }
{  Copyright 1994 - 1998 by Br. David Carlson                           }
{                                                                       }
{  Description:  This is an imitation of the UNIX head command.  It     }
{  displays the first 10 lines of the text file(s) given as the last    }
{  command-line parameter(s).  You can also use a -n option before the  }
{  filename(s), where n is a small positive integer.  In such a case,   }
{  head displays n lines (if there are that many, of course).  Note     }
{  that when more than 1 file is listed on the command line, each       }
{  section of output will be preceded by a message line showing the     }
{  name of the file.                                                    }
{                                                                       }
{  Syntax:                                                              }
{     HEAD filename                                                     }
{     HEAD filename filename filename ...                               }
{     HEAD -n filename                                                  }
{     HEAD -n filename filename filename ...                            }
{     command | HEAD                                                    }
{     command | HEAD -n                                                 }
{  The last 2 examples show that HEAD can accept input piped to it from }
{  another DOS command.                                                 }
{                                                                       }
{  This program is provided as freeware.  See the accompanying readme   }
{  for details.  The software is also provided "as is", without any     }
{  warranty.  The user assumes all of the risk in using the program.    }

const
   DefaultLength = 10;

var
   k, LineCount, CurrentFile, ErrorCode: integer;
   CurrentParam, line: string[80];
   source: text;
   StandardInput, MultipleFiles: boolean;


procedure HandleError;

{  Given:   Nothing.                                                   }
{  Task:    Prints message showing correct use of HEAD and halts.      }
{  Return:  Nothing.                                                   }

   begin
      writeln(#7'Correct syntax:');
      writeln('     HEAD filename');
      writeln('     HEAD filename filename filename ...');
      writeln('     HEAD -n filename');
      writeln('     HEAD -n filename filename filename ...');
      writeln('     command | HEAD');
      writeln('     command | HEAD -n');
      writeln('where n is a small positive integer.');
      halt(1)
   end;


begin
   StandardInput := false;
   MultipleFiles := false;
   LineCount := DefaultLength;
   CurrentFile := 1;

   if ParamCount = 0 then   { probably piped input }
      StandardInput := true
   else
      begin
         CurrentParam := ParamStr(1);

         if CurrentParam[1] = '-' then
            begin
               if ParamCount = 1 then
                  StandardInput := true
               else if ParamCount > 2 then
                  MultipleFiles := true;

               CurrentFile := 2;
               val(CurrentParam, LineCount, ErrorCode);

               if ErrorCode = 0 then
                  begin
                     LineCount := -LineCount;
                     if LineCount <= 0 then
                        begin
                           writeln('ERROR in number.');
                           HandleError
                        end
                  end
               else
                  begin
                     writeln('ERROR in number.');
                     HandleError
                  end
            end
         else if ParamCount > 1 then
            MultipleFiles := true
      end;

   while (StandardInput) or (CurrentFile <= ParamCount) do
      begin
         if StandardInput then
            begin
               assign(source, '');
               StandardInput := false   { to stop the loop next time }
            end
         else
            begin
               if MultipleFiles then
                  writeln('==> ', ParamStr(CurrentFile), ' <==');
               assign(source, ParamStr(CurrentFile));
               CurrentFile := CurrentFile + 1
            end;

         {$I-}
         reset(source);
         {$I+}
         if IOResult <> 0 then
            begin
               writeln(#7'ERROR -- cannot open file.');
               halt(1)
            end;

         k := 0;
         while (not eof(source)) and (k < LineCount) do
            begin
               readln(source, line);
               writeln(line);
               k := k + 1
            end;

         close(source)
      end
end.
